/*
  Name: Base.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy baze (Base).
*/

#include ".\matrix.h"
#include ".\base.h"
#include ".\baseperzistency.h"

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy Base.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Base::Base(void) {
}
/** Destruktor tridy Base.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Base::~Base(void) {
}
/** Metoda createMatrixFromVectors implementuje nasledujici:
  * - z pole vektoru se vytvori matice
  * - vstupni pole je nepredelane na pole vektoru,
  * - vektory jsou v poli reprezentovany jako matice m x n,
  * - vraci matici vektoru
  *
  * @acces public
  * @param **arrayOfVectors -> pole vektoru
  * @param vectorsCount -> pocet vektoru v poli vektoru
  * @return *Matrix -> vraci ukazatel na matici vektoru
 */
Matrix * Base::createMatrixFromVectors(Matrix **arrayOfVectors, int vectorsCount) {
	
	Matrix * matrix = new Matrix();
	Matrix * matrixT = new Matrix();
	Matrix * resultMatrix = new Matrix();
	unsigned int dim = arrayOfVectors[0]->getColumns() * arrayOfVectors[0]->getRows(), i = 0, j = 0;

	resultMatrix->setColumns(dim);
	resultMatrix->setRows(dim);
	// vynulovani vysledne matice
    for( unsigned int k = 0 ; k < resultMatrix->getColumns() ; k++ ) {
        for( unsigned int j = 0 ; j < resultMatrix->getRows() ; j++ ) {
            resultMatrix->setCoordinateAt(j,k,0);
        }
    }
	// rovnou se vytvori normalni i transponovana matice
	matrix->setRows(1);
    matrix->setColumns(dim);
    matrix->setFileName("base.tmp");

    matrixT->setRows(dim);
    matrixT->setColumns(1);
    matrixT->setFileName("base.tmp");
	
	unsigned int col = 0;
	unsigned int row = 0;
    while (row < (unsigned int) vectorsCount) {
    matrixT->setColumns(1);

		for (i=0; i<arrayOfVectors[0]->getRows(); i++) {
			for (j=0; j<arrayOfVectors[0]->getColumns(); j++) {
				matrix->setCoordinateAt(0, col, arrayOfVectors[row]->getCoordinateAt(i,j));
				matrixT->setCoordinateAt(col, 0, arrayOfVectors[row]->getCoordinateAt(i,j));
				col++;
			}
		}
		
		// nasobeni normalni vector * transponovany
		matrixT->multiply(matrix);
		
		// scita se do vysledne matice vsechny hodnoty
		resultMatrix->add(matrixT);
		row++;
		col = 0;
	}
	
	// prumer
	for (i = 0; i<dim; i++) {
		for (j=0; j<dim; j++) {
			resultMatrix->setCoordinateAt(i, j, (resultMatrix->getCoordinateAt(i,j)/vectorsCount));
		}
	}
	
	return resultMatrix;
}
/** Metoda vypocita prumernou matici z pole matic.
  *
  * @acces public
  * @param **arrayOfMatrix -> pole matic
  * @param count -> pocet matic v poli
  * @return *Matrix -> vraci ukazatel na prumernou matici
 */
Matrix * Base::countAverageMatrix(Matrix **arrayOfMatrix, unsigned int count) {

	this->averageMatrix = new Matrix();
	double matrixCoordsValue = 0;
	
	this->averageMatrix->setColumns(arrayOfMatrix[0]->getColumns());
	this->averageMatrix->setRows(arrayOfMatrix[0]->getRows());
	
	for(unsigned int row=0; row<=arrayOfMatrix[0]->getRows(); row++) {
		for(unsigned int column=0; column<=arrayOfMatrix[0]->getColumns(); column++) {
            for (unsigned int i=0; i<count; i++) {
				matrixCoordsValue += arrayOfMatrix[i]->getCoordinateAt(row, column);
			}
			this->averageMatrix->setCoordinateAt(row, column, matrixCoordsValue/count);
			matrixCoordsValue = 0;
		}
	}
	
	return this->averageMatrix;
}
/** Metoda vynasobi (a vrati) na matici (vektor cisel) s vlastnimi vektory.
  * Musi byt splneno:
  * - vstupni vektor musi mit stejny pocet 
  *		- sloupcu jako vlastni vektory
  *		- pouze jeden radek (je to vektor)
  *
  * @acces public
  * @param *matrix -> ukazatel na matici (vektor) cisel
  * @param *filename -> nazev souboru s vlatnimi vektory
  * @return *Matrix -> vraci ukazatel na matici (vektor cisel vynaseobenych s vlastnimi vektory)
 */
Matrix * Base::compareMatrixWithBase(Matrix * matrix, char * filename) {
	
	BasePerzistency * bp = new BasePerzistency();
	Matrix *tmp = new Matrix();
    int dim = 0;
    tmp = bp->loadEigenVectors(filename);

    for( unsigned int k = 0 ; k < tmp->getRows() ; k++ ) {
        double tmpSum = 0;
        for( unsigned int l = 0 ; l < tmp->getColumns() ; l++ ) {
            tmpSum += abs(tmp->getCoordinateAt(k,l));
        }
        for( unsigned int l = 0 ; l < tmp->getColumns() ; l++ ) {
            tmp->setCoordinateAt(k,l,abs(tmp->getCoordinateAt(k,l))/tmpSum);
        }
    }


  	Matrix * cVector = new Matrix();
    cVector->setFileName( matrix->getFileName());
    
	cVector->setColumns(tmp->getRows());
	cVector->setRows(1);

	if ((matrix->getRows() != 1) && (matrix->getColumns() != tmp->getColumns())) {
		cerr << "CHYBA (Base::compareMatrixWithBase): Nesouhlasi pocet radku a sloupcu." << endl;
		exit(1);
	}

	for (unsigned int i = 0; i< tmp->getRows(); i++) {
		double sum = 0;
		for(unsigned int j = 0; j<tmp->getColumns(); j++) {
			sum += tmp->getCoordinateAt(i,j) * matrix->getCoordinateAt(0,j);
		}
		cVector->setCoordinateAt(0, i, sum);
	}
	return cVector;
}
/** Metoda pro vytvoreni baze pred zavolanim programu R-project.
  * Metoda provadi:
  * 1) z pole vektoru se vyrobi matice
  * 2) transponuje se
  * 3) uklada se do souboru jako kovariancni matice
  * 4) nad timto souborem se vytvori zdrojovy kod pro Rko
  *    v souboru vectors.r
  *
  * @acces public
  * @param **arrayOfVectorsMatrix -> pole vektoru
  * @param vectorsCount -> pocet matic v poli
  * @param *filename -> ukazatel na nazev souboru
  * @return void
 */
void Base::createBaseBeforeR(Matrix **arrayOfVectors, int vectorsCount, char * filename) {

	BasePerzistency * nB = new BasePerzistency();
    
	this->covariantMatrix = createMatrixFromVectors(arrayOfVectors, vectorsCount);
	this->covariantMatrix->transpose();
	nB->saveMatrix(this->covariantMatrix, filename);
	this->callR(this->covariantMatrix, filename);
}
/** Metoda pro vytvoreni baze po zavolani programu R-project.
  *
  * @acces public
  * @param *matrix -> ukazatel na matici (vektor) cisel
  * @param *filename -> nazev souboru s vlatnimi vektory
  * @return *Matrix -> vraci ukazatel na matici (vektor cisel vynaseobenych s vlastnimi vektory)
 */
Matrix * Base::createBaseAfterR(Matrix * matrix, char * filename) {
	Matrix * vector = this->compareMatrixWithBase(matrix, filename);
	return vector;
}
/** Metoda vytvori soubor vectors.r, ktery se nacte do R-ka.
  *
  * @acces public
  * @param *matrix -> ukazatel na matici
  * @param *filename -> ukazatel na nazev souboru
  * @return void
 */
void Base::callR(Matrix * matrix, char * CMFILENAME) {
	FILE *fp;
	char *file = "vectors.r";
	
	if ((fp = fopen(file, "w")) != NULL) {
		fprintf(fp, "#n: dimenze, m: pocet rad\nn<-%d\nm<-%d\n", matrix->getColumns(), matrix->getRows());
		fprintf(fp, "matrix <- matrix(NA, m, n)\nfl <- file(\"");
		fprintf(fp, CMFILENAME);
		fprintf(fp, "\", \"r\")\n");
		fprintf(fp, "for( i in 1:m) {\n  for( j in 1:n ) {\n");
		fprintf(fp, "matrix[i,j] <- scan( fl, what = double(0), n = 1, sep = \"\\n\",\n");
		fprintf(fp, "quote = \"\" , dec = \".\",\n");
		fprintf(fp, "skip = 0, nlines = 0, na.strings = \"NA\",\n");
		fprintf(fp, "flush = FALSE, fill = FALSE, strip.white = FALSE, quiet = FALSE,\n");
		fprintf(fp, "blank.lines.skip = TRUE, multi.line = TRUE, comment.char = \"\")\n}\n}");
        fclose(fp);
    }
	if ((fp = fopen("pca.r", "w")) != NULL) {
		fprintf(fp, "\npca <- prcomp(matrix, retx = FALSE)\n");
		fprintf(fp, "write(pca$sdev, \"d:\\eval_pca.txt\", append = FALSE)\n");
		fprintf(fp, "write(pca$rotation, \"eigen.dat\", append = FALSE)\nrm(pca)");
		fclose(fp);
	}
}
/** Metoda pro nastaveni matice vlastnich vektoru.
  *
  * @acces public
  * @param *matrix -> ukazatel na matici vlastnich vektoru
  * @return void
 */
void Base::setEigenVectors(Matrix * matrix){
	this->eigenVectors = matrix;
}
/** Metoda pro ziskani matice vlastnich vektoru.
  *
  * @acces public
  * @param void
  * @return Matrix* -> vraci ukazatel na matici vlastnich vektoru
 */
Matrix * Base::getEigenVectors() {
	return this->eigenVectors;
}
