/*
  Name: BasePerzistency.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy BasePerzistency.
*/

#include ".\baseperzistency.h"
#include ".\matrix.h"

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy BasePerzistency.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
BasePerzistency::BasePerzistency(void) {
}
/** Destruktor tridy BasePerzistency.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
BasePerzistency::~BasePerzistency(void) {
}
/** Metoda pro nacteni souboru s vlastnimi vektory, tj. nacte se baze.
  * @version 1.2.2004
  *
  * @acces public
  * @param *filename -> ukazatel na nazev souboru s vlastnimi vektory
  * @return Base* -> ukazatel na nactenou bazi
 */
Base * BasePerzistency::loadBase(char * filename) {
	if ((this->loadEigenVectors(filename)) == NULL) {
        return NULL;
	} else {
    	Base *b = new Base();
        b->setEigenVectors(this->loadEigenVectors(filename));
        return b;
	}
}

/** Metoda pro nacteni vlastnich vektoru ze souboru.
  * Nacita se pouze prvnich 20 vektoru. Toto cislo bylo
  * zjisteno na zaklade experimentu.
  * @version 1.2.2004
  *
  * @acces public
  * @param *EVFILENAME -> ukazatel na nazev souboru s vlastnimi vektory
  * @return Matrix* -> ukazatel na matici, kde kazdy radek predstavuje 1 vektor
 */
Matrix * BasePerzistency::loadEigenVectors(char * EVFILENAME) {
 	FILE *fp;
	Matrix *eigenVectors = new Matrix();
    eigenVectors->setFileName(EVFILENAME);
	unsigned int rows = 0;
	double value = 0;
    unsigned int eigenCount = 0;
	double kos = 0;
    unsigned int rowCount = 0, colCount = 0;
	
    if ( (fp = fopen("eval_pca.txt", "r")) == NULL ) {
		//cerr << "CHYBA (BasePerzistency::loadEigenVectors): Nelze otevrit soubor \"eval_pca.txt\" pro cteni." << endl;
        return NULL;
    } 

    while (!feof(fp)) {
        fscanf(fp, "%lf", &kos);
        eigenCount++;
    }
    eigenCount--;

    fclose(fp);
    
    /* tady je mozne pouzit tu chytrou metodu na ty vlastni cisla */
    if ( (fp = fopen(EVFILENAME, "r")) == NULL ) {
		return NULL;
    } else {
		eigenVectors->setColumns(eigenCount);
		eigenVectors->setRows(20); // pocet vlastnich vektoru, poznatky na zaklade experimetnu

		while (rowCount < 20) {
			colCount = 0;
			while ((colCount < (unsigned int)eigenCount)){
				fscanf(fp, "%lf", &value);
				eigenVectors->setCoordinateAt(rowCount, colCount, value);
				colCount++;
			}
			rowCount++;
		}
		
		fclose(fp);
	}

	return eigenVectors;
}
/** Metoda pro ukladani matice do souboru;
  * kazdy provek matice se uklada na novy radek.
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na ukladanou matici
  * @param *CMFILENAME -> ukazatel na nazev souboru s kovariancni matici
  * @return void
 */
void BasePerzistency::saveMatrix(Matrix *matrix, char * CMFILENAME) {
	FILE *fp;

	if ( (fp = fopen(CMFILENAME, "w")) == NULL ) {
		cerr << "CHYBA (BasePerzistency::saveMatrix): Pri otevirani souboru (" << CMFILENAME << ") doslo k chybe." << endl; 
		exit(1);
	}
	else {
		for (unsigned int i=0; i<matrix->getRows(); i++) {
			for (unsigned int j=0; j<matrix->getColumns(); j++) {
				if (j == matrix->getColumns() - 1) {
					// pro Rko se uklada kazda hodnota matice na novy radek
					fprintf(fp, "%lf\n", matrix->getCoordinateAt(i, j));
				} else {
					// pro Rko se uklada kazda hodnota matice na novy radek
					fprintf(fp, "%lf\n", matrix->getCoordinateAt(i, j));
				}
			}
		}
		
		fclose(fp);
	}
}
/** Metoda pro nastaveni nazvu souboru kovariancni matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param *newFile -> ukazatel na soubor
  * @return void
 */
void BasePerzistency::setCMfilename(char * newFile) {
	this->CMfilename = newFile;
}
/** Metoda pro navraceni nazvu souboru kovariancni matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return char* -> vraci ukazatel na jmeno souboru
 */
char * BasePerzistency::getCMfilename(void) {
	return this->CMfilename;
}
/** Metoda pro nastaveni nazvu souboru s vlastnimi vektory.
  * @version 1.2.2004
  *
  * @acces public
  * @param *newFile -> ukazatel na soubor
  * @return void
 */
void BasePerzistency::setEVfilename(char * newFile) {
	this->EVfilename = newFile;
}
/** Metoda pro navraceni nazvu souboru s vlastnimi vektory.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return char* -> vraci ukazatel na jmeno souboru
 */
char * BasePerzistency::getEVfilename(void) {
	return this->EVfilename;
}
