/*
  Name: Database.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy databaze (Database).
*/

#include ".\database.h"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy Database.
  * Vynuluje vsechny privatni prvky tridy.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Database::Database() {
	this -> matrixValues = NULL;
	this -> metric = NULL;
	this -> countValues = 0;
}
/** Destruktor tridy Database.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Database::~Database(void) {
}
/** Metoda, ktera prida do pole matic dalsi matici typu Matix.
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na matici, ktera je pridavana do pole matic
  * @return void
 */
void Database::addMatrix(Matrix *matrix) {
	size_t size;

	if (this -> matrixValues != NULL) {
		size = _msize(this -> matrixValues);
	} // end if

	if (this -> countValues == 0) {

		if ((this -> matrixValues = (Matrix *)malloc(sizeof(Matrix))) != NULL) {
			this -> countValues++;
			this -> matrixValues[this -> countValues - 1] = *matrix;
		} else {
			cerr << "CHYBA (Database::addMatrix): Nepodarilo se alokovat pamet!" << endl;
			exit(1);
		} // end if

	} else {

		if ((this -> matrixValues = (Matrix *)realloc(this -> matrixValues, size + sizeof(Matrix))) != NULL) {
			this -> countValues++;
			this -> matrixValues[this -> countValues - 1] = *matrix;
		} else {
			cerr << "CHYBA (Database::addMatrix): Nepodarilo se prealokovat pamet!" << endl;
			exit(1);
		} // end if
	} // end if
}

/** Metoda, ktera odebere matici z pole matic.
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na matici, ktera se ma odebrat
  * @return void
 */
void Database::removeMatrix(Matrix *matrix) {
	// pomocna promenna, ktera obsahuje pozici matice v poli, ktera se bude odebirat
	int removingMatrix = -1;

	for(unsigned int i = 0; i < this -> countValues; i++) {

		if ((matrix -> getRows() == this -> matrixValues[i].getRows())&&(matrix -> getColumns() == this -> matrixValues[i].getColumns())) {
			// pomocna promenna, ktera urcuje, zda se ma matice odebrat (1) nebo ne (0)
			unsigned int removed = 1;

			for(unsigned int j = 0; j < matrix -> getRows(); j++) {
				for(unsigned int k = 0; k < matrix -> getColumns(); k++) {
					if (matrix -> getCoordinateAt(j, k) == this -> matrixValues[i].getCoordinateAt(j, k)) {
						removed = 1;
					} else {
						removed = 0;
						break;
					}
				} // end for

				if (removed == 0) {
					break;
				} // end if
			} // end for

			if (removed == 1) {
				removingMatrix = i;
				break;
			} // end if
		} // end if

	} // end for

	if (removingMatrix != -1) {
		for(i = removingMatrix; i < this -> countValues - 1; i++) {
			this -> matrixValues[i] = this -> matrixValues[i + 1];
		} // end for

		if (this -> countValues == 1) {
			this -> matrixValues = NULL;
			this -> countValues--;
		} else {
			size_t size = _msize(this -> matrixValues);
			if ((this -> matrixValues = (Matrix *)realloc(this -> matrixValues, size - sizeof(Matrix))) != NULL) {
				this -> countValues--;
			} else {
				cerr << "CHYBA (Database::removeMatrix): Nepodarilo se prealokovat pamet!" << endl;
				exit(1);
			} // end if
		} // end if
	} // end if
}
/** Metoda, ktera vybere podobne matice z pole matic.
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na matici, podle ktere se budou vybirat matice z pole matic
  * @param trashold -> odpad
  * @param count -> parametr, ve kterem se bude vracet pocet vybranych matic
  * @return **Matrix -> vraci pole vybranych matic
 */
Matrix ** Database::selectMatrix(Matrix *matrix, float trashold, unsigned int &count) {
	Matrix **selectedMatrix = NULL;
	count = 0;

	if (this -> metric != NULL) {

		for(unsigned int i = 0; i < this -> countValues; i++) {
			if ((matrix -> getRows() == this -> matrixValues[i].getRows())&&(matrix -> getColumns() == this -> matrixValues[i].getColumns())) {

				double tmpValue = this -> metric -> getDistance(matrix, &matrixValues[i]);
                

				if (tmpValue >= trashold ) {

					if (selectedMatrix != NULL) {
						size_t size = _msize(selectedMatrix);
						if ((selectedMatrix = (Matrix **)realloc(selectedMatrix, size + sizeof(Matrix *))) != NULL) {
							selectedMatrix[count] = &matrixValues[i];
							count++;
						} else {
							cerr << "CHYBA (Database::selectMatrix): Nepodarilo se prealokovat pamet!" << endl;
							exit(1);
						}
					} else {
						if ((selectedMatrix = (Matrix **)malloc(sizeof(Matrix *))) != NULL) {
							selectedMatrix[count] = &matrixValues[i];
							count++;
						} else {
							cerr << "CHYBA (Database::selectMatrix): Nepodarilo se alokovat pamet!" << endl;
							exit(1);
						} // end if
					} // end if

				} // end if

			} // end if
		} // end for

	} else {

		cerr << "CHYBA (Database::selectMatrix): Neni nastavena metrika. Nelze vybirat podobne matice!" << endl;
		exit(1);
	}

	return selectedMatrix;
}
/** Metoda pro nastaveni pouzivane metriky.
  * @version 1.2.2004
  *
  * @acces public
  * @param *metric -> ukazatel na nastavovanou metriku
  * @return void
 */
void Database::setMetric(Metric *metric) {
	this -> metric = metric;
}
/** Metoda, ktera vrati pocet matic v poli.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return unsigned int -> pocet matic v poli
 */
unsigned int Database::getCountValues(void) {
	return this -> countValues;
}
/** Metoda, ktera vraci ID metriky.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return int -> ID metriky
 */
int Database::getMetricId(void) {
	if (this -> metric == NULL) {
		return -1;
	} // end if

	return this -> metric -> getId();
}
/** Metoda, ktera vraci ukazatel na matici podle zadane pozice.
  * @version 1.2.2004
  *
  * @acces public
  * @param position -> pozice matice v poli
  * @return Matrix* -> ukazatel na matici v poli matic
 */
Matrix * Database::getMatrix(unsigned int position) {
	return &matrixValues[position];
}
