/*
  Name: DatabasePerzistency.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy DatabasePerzistency.
*/

#include ".\databaseperzistency.h"

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy DatabasePerzistency.
  * Uchovava nazev souboru, ve kterem je ulozena databaze.
  * @version 1.2.2004
  *
  * @acces public
  * @param *fileName -> ukazatel na nazev souboru, ve kterem je ulozena databaze
  * @return void
 */
DatabasePerzistency::DatabasePerzistency(char *fileName) {
	this -> fileName = fileName;
}
/** Destruktor tridy DatabasePerzistency.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
DatabasePerzistency::~DatabasePerzistency(void) {
}
/** Metoda slouzici pro ulozeni database do souboru.
  * @version 1.2.2004
  *
  * @acces public
  * @param *database -> ukazatel na databazi, ktera se ma ulozit do souboru
  * @return void
 */
void DatabasePerzistency::saveDatabase(Database *database) {
	FILE *file;

	file = fopen(this -> fileName, "w");
	if (file != NULL) {
		// ulozeni ID metriky
		fprintf(file, "%i\n", database -> getMetricId());
		// ulozeni poctu matic v databazi
		fprintf(file, "%i\n", database -> getCountValues());

		if (database -> getCountValues() > 0) {

			for(unsigned int i = 0; i < database -> getCountValues(); i++) {
				// ulozeni nazvu souboru matice
				fprintf(file, "%s\n", database -> getMatrix(i) -> getFileName());
				// ulozeni poctu radku a sloupcu matice
				fprintf(file, "%i %i\n", database -> getMatrix(i) -> getRows(), database -> getMatrix(i) -> getColumns());

				for(unsigned int j = 0; j < database -> getMatrix(i) -> getRows(); j++) {
					for(unsigned int k = 0; k < database -> getMatrix(i) -> getColumns(); k++) {
						if (k != database -> getMatrix(i) -> getColumns() - 1) {
							fprintf(file, "%lf ", database -> getMatrix(i) -> getCoordinateAt(j, k));
						} else {
							fprintf(file, "%lf\n", database -> getMatrix(i) -> getCoordinateAt(j, k));
						} // end if
					} // end for
				} // end for

			} // end for

		} // end if
		fclose(file);
	} else { // nepodarilo se soubor otevrit
		cerr << "CHYBA (DatabasePerzistency::saveDatabase): Nepodarilo se otevrit soubor pro zapis!" << endl;
	} // end if
}
/** Metoda slouzici pro nacteni databaze ze souboru.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return *Database -> vraci se ukazatel na instanci tridy Datababase
 */
Database * DatabasePerzistency::loadDatabase(void) {

	FILE *file;

	file = fopen(this -> fileName, "r");
	if (file != NULL) {
		unsigned int temp = 0;
		Database *database = new Database();
		// nacteni ID metriky
		fscanf(file, "%i", &temp);

		Metric *metric;
		switch (temp) {
				// vytvorena instance euklidovy metriky
			case 1: metric = new EuclidsMetric();
					break;
				// vytvorena instance kosinovy metriky
			case 2: metric = new CosineMetric();
					break;
		}
		database -> setMetric(metric);
		// nacteni poctu matic
		fscanf(file, "%i", &temp);

		for(unsigned int i = 0; i < temp; i++) {
			unsigned int rows = 0, columns = 0;
			double value = 0;

			char *fileName, pom[1024];
			fscanf(file, "%s", pom);
			fileName = (char *)malloc(strlen(pom) * sizeof(char));
			strcpy(fileName, pom);

			// nacteni poctu radku a sloupcu matice
			fscanf(file, "%i%i", &rows, &columns);

			Matrix *matrix = new Matrix();
			matrix -> setFileName(fileName);
			matrix -> setRows(rows);
			matrix -> setColumns(columns);

			for(unsigned int j = 0; j < rows; j++) {
				for(unsigned int k = 0; k < columns; k++) {
					// nacteni hodnoty matice na prislusne pozici
					fscanf(file, "%lf", &value);
					matrix -> setCoordinateAt(j, k, value);
				} // end for
			} // end for
			
			database -> addMatrix(matrix);
		} // end for

		fclose(file);

		return database;
	} else {
		// nepodarilo se otevrit soubor, proto se vraci NULL
		return NULL;
	} // end if
}
