/*
  Name: EuclidsMetric.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy euklidovy metriky (EuclidsMetric).
*/

#include ".\euclidsmetric.h"

#include <stdio.h>
#include <math.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy EuclidsMetric.
  * Nastavuje pocatecni hodnoty privatnich promennych.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
EuclidsMetric::EuclidsMetric(void) {
	this -> id = 1;
}
/** Destruktor tridy EuclidsMetric.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
EuclidsMetric::~EuclidsMetric(void) {
}
/** Metoda, ktera slouzi pro vypocet euklidovy metriky.
  *
  * pozn.:
  * Vektor je predstavovan matici, ktera ma bud jeden sloupec nebo jeden radek (obvykle to byva radek).
  * !!! Matice musi mit stejny pocet radku i sloupcu !!!
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrixA -> ukazatel na matici, ktera se bude pouzita pri vypoctu metriky
  * @param *matrixB -> ukazatel na matici, ktera se bude pouzita pri vypoctu metriky
  * @return float -> vraci se hodnota vypoctene metriky
 */
double EuclidsMetric::getDistance(Matrix *matrixA, Matrix *matrixB) {
	// Porovnani, zda obe matice maji shodny pocet radku a sloupcu
	// a jestli radek nebo sloupec je jednotkovy (tzn. matice je vektor)...
	if ((matrixA -> getRows() == matrixB -> getRows())&&(matrixA -> getColumns() == matrixB -> getColumns())
        &&((matrixA -> getRows() == 1)||(matrixA -> getColumns() == 1))) {

		double pom = 0;

		for(unsigned int i = 0; i < matrixA -> getRows(); i++) {
			for(unsigned int j = 0; j < matrixA -> getColumns(); j++) {
				double pomm = 0;
				pomm = matrixA -> getCoordinateAt(i, j) - matrixB -> getCoordinateAt(i, j);
				pom += (pomm * pomm);
			} // end for
		} // end for

		pom = 1-sqrt(pom);

		return pom;

	} else {
		cerr << "CHYBA (EuclidsMetric::getDistance): Matice nemaji shodny pocet radku a sloupcu nebo nemaji jeden radek nebo sloupce." << endl;
		exit(1);
        return NULL;
	} // end if
}
/** Metoda vrati ID metriky.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return unsigned int -> vraci se ID metriky
 */
unsigned int EuclidsMetric::getId(void) {
	return this -> id;
}
