/*
  Name: Matrix.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy matice (Matrix).
*/

#include ".\matrix.h"

#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include <string.h>
#include <iostream>

using namespace std;

/** Konstruktor tridy Matrix.
  * Vynuluje vsechny privatni prvky tridy.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Matrix::Matrix(void) {
	this -> columns = 0;
	this -> rows = 0;
	this -> fileName = NULL;
	this -> matrixValues = NULL;
}
/** Destruktor tridy Matrix.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
Matrix::~Matrix(void) {
}
/** Metoda, ktera vraci pocet sloupcu matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return unsigned int -> vraci pocet sloupcu matice
 */
unsigned int Matrix::getColumns(void) {
	return this -> columns;
}
/** Metoda, ktera vraci pocet radku matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return unsigned int -> vraci pocer radku matice
 */
unsigned int Matrix::getRows(void) {
	return this -> rows;
}
/** Metoda, ktera nastavi pocet sloupcu matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param columns -> pocet nastavovanych sloupcu matice
  * @return void
 */
void Matrix::setColumns(unsigned int columns) {
	this -> columns = columns;
	this -> allocateMatrix();
}
/** Metoda, ktera nastavi pocet radku matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param rows -> pocet nastavovanych radku matice
  * @return void
 */
void Matrix::setRows(unsigned int rows) {
	this -> rows = rows;
	this -> allocateMatrix();
}
/** Metoda, ktera vraci ukazatel na nazev souboru,
  * ve kterem je ulozen puvodni obrazek.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return char* -> ukazatel na nazev souboru s puvodnim obrazkem
 */
char * Matrix::getFileName(void) {
	return this -> fileName;
}
/** Metoda, ktera nastavi ukazatel na nazev souboru,
  * ve kterem je ulozen puvodni obrazek.
  * @version 1.2.2004
  *
  * @acces public
  * @param fileName -> ukazatel na nazev souboru s puvodnim obrazkem
  * @return void
 */
void Matrix::setFileName(char * fileName) {
	this -> fileName = fileName;
	
}
/** Metoda pro ulozeni hodnoty v matici na prislusne misto.
  * @version 1.2.2004
  *
  * @acces public
  * @param row -> cislo radku v matici
  * @param column -> cislo sloupce v matici
  * @param value -> ukladana hodnota
  * @return void
 */
void Matrix::setCoordinateAt(unsigned int row, unsigned int column, double value) {
	if ((column < this -> getColumns())&&(row < this -> getRows())) {
		this -> matrixValues[row][column] = value;
	} // end if
}
/** Metoda pro vybrani hodnoty z prislusneho mista v matici.
  * @version 1.2.2004
  *
  * @acces public
  * @param row -> cislo radku v matici
  * @param column -> cislo sloupce v matici
  * @return double -> vracena hodnota z prislusneho mista v matici
 */
double Matrix::getCoordinateAt(unsigned int row, unsigned int column) {
	// kontrola, zda nesahame mimo alokovanou hodnotu v pameti
	if ((column < this -> getColumns())&&(row < this -> getRows())) {
		return this -> matrixValues[row][column];
	} else {
		// jinak se vrati NULL
		return NULL;
	} // end if
}
/** Metoda, ktera pricte k jedne matici matici druhou.
  * !!! Obe matice musi mit shodny pocet
  * radku a sloupcu, jinak nelze scitat !!!
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na pricitanou matici
  * @return void
 */
void Matrix::add(Matrix *matrix) {
	// kontrola, zda maji obe matice shodny pocet radku i sloupcu
	if ((this -> getColumns() == matrix -> getColumns())&&(this -> getRows() == matrix -> getRows())) {
		for(unsigned int i = 0; i < this -> getRows(); i++) {
			for(unsigned int j = 0; j < this -> getColumns(); j++) {
				this -> setCoordinateAt(i, j, this -> getCoordinateAt(i, j) + matrix -> getCoordinateAt(i, j));
			} // end for
		} // end for
	} else {
		cerr << "CHYBA (Matrix::add): Matice nemaji shodny pocet radku a sloupcu! Matice nelze scitat." << endl;
		exit(1);
	} // end if
}

/** Metoda pro vynasobeni matice skalarem.
  * !!!Pretizena metoda!!!
  * @version 1.2.2004
  *
  * @acces public
  * @param scalar -> skalar pro vynasobeni matice
  * @return void
 */
void Matrix::multiply(double scalar) {
	for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
			this -> setCoordinateAt(i, j, scalar * this -> getCoordinateAt(i, j));
		} // end for
	} // end for
}
/** Metoda pro vynasobeni matice matici.
  * !!!Pretizena metoda!!!
  * !!! pocet sloupcu teto matice === poctu radku matice v argumentu metody !!!
  * @version 1.2.2004
  *
  * @acces public
  * @param *matrix -> ukazatel na matici, se kterou se bude nasobit
  * @return void
 */
void Matrix::multiply(Matrix *matrix) {
	// tato matice musi mit stejny pocet sloupcu jako ma matice v argumentu metody
	if (this -> getColumns() == matrix -> getRows()) {
		// pomocna matice, kde se bude ukladat vynasobeni
		// ma velikost rows_1 x columns_2
		Matrix *m = new Matrix();
		m -> setRows(this -> getRows());
		m -> setColumns(matrix -> getColumns());
		
		for(unsigned int i = 0; i < m -> getRows(); i++) {
			for(unsigned int j = 0; j < m -> getColumns(); j++) {
				double pom = 0;

				for(unsigned int k = 0; k < this -> getColumns(); k++) {
					pom += this -> getCoordinateAt(i, k) * matrix -> getCoordinateAt(k, j);
				} // end for

				m -> setCoordinateAt(i, j, pom);
			} // end for
		} // end for

		this -> setRows(m -> getRows());
		this -> setColumns(m -> getColumns());

		for(unsigned int i = 0; i < this -> getRows(); i++) {
			for(unsigned int j = 0; j < this -> getColumns(); j++) {
				this -> setCoordinateAt(i, j, m -> getCoordinateAt(i, j));
			} // end for
		} // end for
	} else {
		cerr << "CHYBA (Matrix::multiply): Matice nebyly vynasobeny! Pocet sloupcu jedne matice se nerovna poctu radku druhe matice." << endl;
		exit(1);
	} // end if
}
/** Metoda pro transponovani matice.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
void Matrix::transpose(void) {
	Matrix *m = new Matrix();
	m -> setRows(this -> getColumns());
	m -> setColumns(this -> getRows());

	for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
			m -> setCoordinateAt(j, i, this -> getCoordinateAt(i, j));
		} // end for
	} // end for

	this -> setRows(m -> getRows());
	this -> setColumns(m -> getColumns());

	for(unsigned int i = 0; i < m -> getRows(); i++) {
		for(unsigned int j = 0; j < m -> getColumns(); j++) {
			this -> setCoordinateAt(i, j, m -> getCoordinateAt(i, j));
		} // end for
	} // end for
}
/** Metoda pro prevzorkovani obrazku ulozeneho v matici.
  * Implementovana metodou bilinearniho prevzorkovani.
  * 
  * @version 11.1.2004
 */
void Matrix::resample(void) {
	Matrix *m = new Matrix();

	m -> setRows(13);
	m -> setColumns(13);

	double zoom1 = m -> getRows() / (double) this -> getRows();
	double zoom2 = m -> getColumns() / (double) this -> getColumns();

	for(unsigned y = 0; y < m -> getRows(); y++) {
		for(unsigned x = 0; x < m -> getColumns(); x++) {
			double xr = x / zoom2;
			double yr = y / zoom1;
			
			int rxr = (int)xr;
			if ((xr - rxr) >= 0.5)
				rxr++;

			int ryr = (int)yr;
			if ((yr - ryr) >= 0.5)
				ryr++;

			double ncolor = 0;

			if (((double)rxr == xr)&&((double)ryr == yr)) {
				ncolor = this -> getCoordinateAt(ryr, rxr);
			} else {

				int j = (int)xr, k = (int)yr;

				if ((xr - j) >= 0.5)
					j++;
				if ((yr - k) >= 0.5)
					k++;

				double a = xr - j;
				double b = yr - k;
				double dest = (1 - a) * (1 - b) * this -> getCoordinateAt(k, j) +
					a * (1 - b) * this -> getCoordinateAt(k, j + 1) +
					b * (1 - a) * this -> getCoordinateAt(k + 1, j) +
					a * b * this -> getCoordinateAt(k + 1, j + 1);


				int rncolor = (int)dest;
				if (rncolor < 0)
					rncolor = -rncolor;
				if ((dest - rncolor) >= 0.5)
					rncolor += 1;
				if (rncolor > 255)
					rncolor = 255;
				ncolor = (double)rncolor;
			}

			m -> setCoordinateAt(y, x, ncolor);
		}
	}

	this -> setColumns(m -> getColumns());
	this -> setRows(m -> getRows());
	for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
			this -> setCoordinateAt(i, j, m -> getCoordinateAt(i, j));
		}
	}

}
/** Metoda, ktera vytiskne matici na obrazovku.
  * Slouzi pro ucely testovani.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
void Matrix::print(void) {
	//cout << "Tisk matice (" << this -> getRows() << ", " << this -> getColumns() << "):" << endl;
    cout << "Soubor " << this -> fileName << endl;
	
/*    for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
			cout << this -> getCoordinateAt(i, j) << " ";
		}
		cout << endl;
	}*/
}

/** Metoda k normovani vektoru.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
void Matrix::normalize(void) {
	double suma = 0;

	for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
            suma += abs(this -> getCoordinateAt(i, j));
        } // end for
	} // end for


	for(unsigned int i = 0; i < this -> getRows(); i++) {
		for(unsigned int j = 0; j < this -> getColumns(); j++) {
			this -> setCoordinateAt(i, j, abs(this -> getCoordinateAt(i, j)) / suma);
		} // end for
	} // end for
    

    

}
/** Metoda, ktera vrati viceradkovou jako jednoradkovou matici (tzn. vektor).
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return Matrix* -> vraci ukazatel na jednoradkovou matici (vektor)
 */
Matrix * Matrix::getAsVector(void){
    
    Matrix * tmp = new Matrix();

    tmp -> setFileName(this->getFileName());
    tmp -> setRows(1);
    tmp -> setColumns(this->getRows() * this->getColumns());
    
    unsigned int col = 0;

	for (unsigned int i=0; i< this->getRows(); i++) {
		for (unsigned int j=0; j<this->getColumns(); j++) {
			tmp->setCoordinateAt(0, col, this->getCoordinateAt(i,j));
			col++;
		} // end for
	} // end for

    return tmp;
}
/** Metoda, ktera alokuje pamet pro matici, pripadne pamet realokuje.
  * @version 1.2.2004
  *
  * @acces private
  * @param void
  * @return unsigned int -> vraci kladne cislo v pripade uspechu, jinak NULL
 */
unsigned int Matrix::allocateMatrix(void) {
	// kontrola, zda jiz byla matice alokovana
	if (this -> matrixValues == NULL) { // zde se pamet alokuje

		if ((this -> getColumns() > 0)&&(this -> getRows() > 0)) {
			if ((this -> matrixValues = (double **)malloc(sizeof(double *) * this -> getRows())) != NULL) {

				for(unsigned int i = 0; i < this -> getRows(); i++) {
					if ((this -> matrixValues[i] = (double *)malloc(sizeof(double) * this -> getColumns())) == NULL) {
						cerr << "CHYBA (Matrix::allocateMatrix): Nepodarilo se alokovat pamet." << endl;
						exit(1);

						return NULL;

					} // end if
				} // end for

				return sizeof(this -> matrixValues);
			} else {
				cerr << "CHYBA (Matrix::allocateMatrix): Nepodarilo se alokovat pamet." << endl;
				exit(1);

				return NULL;
			} // end if
		} else {
			return NULL;
		} // end if

	} else { // zde se pamet prealokuje

		if ((this -> getRows() == 0)||(this -> getColumns() == 0)) {
			size_t size1, poc1;
			size1 = _msize(this -> matrixValues);
			poc1 = size1 / sizeof(double *);
			for(unsigned int i = 0; i < poc1; i++) {
				this -> matrixValues[i] = NULL;
			} // end for
			this -> matrixValues = NULL;

			return NULL;
		} else {
			size_t size1, size2, poc1, poc2, pre1, pre2;
			size1 = _msize(this -> matrixValues);
			poc1 = size1 / sizeof(double *);

			// je-li pocet radku mensi nez pocet radku v pameti, musime nadbivajici radky odebrat
			if (this -> getRows() < poc1) {
				for(unsigned int i = this -> getRows(); i < poc1; i++) {
					this -> matrixValues[i] = NULL;
				} // end for
			} // end if

			// kdyz se pocet radku nerovna staremu poctu, prealokujeme pocet radku
			if (this -> getRows() != poc1) {
				// nastaveni o kolik radku se ma pamet prealokovat
				pre1 = this -> getRows() - poc1;

				if ((this -> matrixValues = (double **)realloc(this -> matrixValues, size1 + (pre1 * sizeof(double *)))) == NULL) {
					cerr << "CHYBA (Matrix::allocateMatrix): Nepodarilo se prealokovat pamet." << endl;
					exit(1);
					return NULL;
				} // end if
			} // end if

			for(unsigned int i = 0; i < this -> getRows(); i++) {

				if (i < poc1) {
					size2 = _msize(this -> matrixValues[i]);
					poc2 = size2 / sizeof(double);
					pre2 = this -> getColumns() - poc2;
					if ((this -> matrixValues[i] = (double *)realloc(this -> matrixValues[i], size2 + (pre2 * sizeof(double)))) == NULL) {
						cerr << "CHYBA (Matrix::allocateMatrix): Nepodarilo se prealokovat pamet." << endl;
						exit(1);
						return NULL;
					} // end if
				} else {
					if ((this -> matrixValues[i] = (double *)malloc(this -> getColumns() * sizeof(double))) == NULL) {
						cerr << "CHYBA (Matrix::allocateMatrix): Nepodarilo se alokovat pamet." << endl;
						exit(1);
						return NULL;
					} // end if
				} // end if

			} // end for

			return sizeof(this -> matrixValues);
		} // end if

	} // end if
}
