/*
  Name: UserInterface.cpp
  Authors: Vlastimil Hopjan, hop007, hopjan@luvasoft.com
           David Pilar,      pil032, pilar.david@seznam.cz
		   Jiri Sovadina,    sov015, ji.so@post.cz
		   Marek Vojtek,     voj086, old-shatterhand@email.cz
  Date: 1.2.2004
  Description: Implementace tridy uzivatelskeho rozhrani (UserInterface).
*/

#include ".\userinterface.h"

#include <iostream>
#include <string>

using namespace std;

/** Konstruktor tridy UserInterface.
  * Nastavi vsechny privatni prvky na pocatecni hodnoty.
  * @version 1.2.2004
  *
  * @acces public
  * @param argCount -> pocet vstupnich argumentu
  * @param **argValues -> vstupni argumenty
  * @return void
 */
UserInterface::UserInterface(int argCount, char ** argValues) {
	this->argCount = argCount;
	this->argValues = argValues;
    this->m_pr = new PictureReader();
    this->m_bp = new BasePerzistency();
}	
/** Destruktor tridy UserInterface.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
UserInterface::~UserInterface() {
}
/** Metoda pro vypis napovedy.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
void UserInterface::printHelp() {
	cout << "Program pro porovnavani obliceju" << endl;
	cout << "Autori: Hopjan, Pilar, Sovadina, Vojtek" << endl;
	cout << "Syntaxe prikazu:" << endl << endl;
	cout << "faces --help\t\t\tvypise tuto napovedu" << endl << endl;
	cout << "faces --insert picture.pgm\tvlozi obrazek do databaze" << endl;
	cout << endl;
	cout << "faces --create picture1.pgm picture1.pgm ... " << endl;
 	cout << "\t\t\t\tvytvori bazi pro";
 	cout << " vypocet PCA ze zadanych obrazku" << endl;
 	cout << "faces --db dbname\t\turci databazi, se kterou se bude";
  	cout << " pracovat" << endl;
  	cout << "faces --find picture.pgm\tvyhleda obrazky, ktere jsou" << endl;
   	cout << "\t\t\t\tzadanemu obrazku podobne" << endl;
   	cout << "faces --base basename\t\turci, ktera base bude pouzita" << endl;
    cout << "\t\t\t\tpri redukci dimenze vektoru" << endl;
   	cout << "Priklady pouziti programu:" << endl << endl;
   	cout << "faces --find picture1.pgm --db database1 --base base" << endl;
   	cout << "\t\t\t\thleda picture1.pgm v databazi database1 a " << endl;
    cout << "\t\t\t\tpro redukci pouzije bazi base" << endl;
   	cout << "faces --create picture1.pgm picture2.pgm --base base" << endl;
   	cout << "\t\t\t\tz picture1.pgm  a picture2.pgm vytvori " << endl;
    cout << "\t\t\t\tbazi base" << endl;
   	cout << "faces --th 0.98" << endl;
   	cout << "\t\t\t\tnastavi minimalni hodnotu vzdalenosti, ktera " << endl;
    cout << "\t\t\t\tbude povazovana za uspesne vyhledani (implicitne 0.85)." << endl;
   	cout << "faces --metric 1" << endl;
    cout << "\t\t\t\tnastavi pouzivanou metriku (euklidova-1-impl.,kosinova-2)" << endl;
}
/** Metoda pro vykonani programu.
  * @version 1.2.2004
  *
  * @acces public
  * @param void
  * @return void
 */
void UserInterface::execute(void){

	int status = 0;
 	int find = 1;
  	int create = 2;
    int insert = 3;
    int substatus = 0;
    int base = 1;
	int i = 1;
    float m_th = (float)0.85;
    PictureReader m_pr;
    int m_metricId = -1;
	int countOfFiles = 0;
    char * fileName = NULL;
	Matrix **m_array;
	Matrix *m;
    Metric *em = new EuclidsMetric();
    Metric *cm = new CosineMetric();
	
	if( ( m_array = (Matrix **)malloc(sizeof(Matrix *) ) ) == NULL ) {
    	cout << "Chyba: Nepodarilo se alokovat pamet.";
        exit(1);
    }
    
    argCount--; // preskocim jmeno souboru

    if( argCount == 0 ) {
        printHelp();
        exit(0);
    }

    while( argCount > 0) {
        if( strcmp( argValues[i],"--help") == 0 ) {
            i++;
            argCount--;
			printHelp();
            if( argCount == 0 ) 
                break;

        }
  		if( strcmp( argValues[i],"--th") == 0 ) {
  			i++;
			argCount--;
            if( (argCount == 0 )|| isSwitch(argValues[i])) {
                cerr << "CHYBA: Musi nasledovat hodnota hranice pro uspesne vyhledani." <<endl;
                exit(1);
            }
            m_th = (float)atof(argValues[i]);
  			i++;
			argCount--;
            if( argCount == 0 ) 
                   break;
		
        }
        if( strcmp( argValues[i],"--metric") == 0 ) {
  			i++;
			argCount--;
            if( (argCount == 0 )|| isSwitch(argValues[i])) {
                cerr << "CHYBA: Musi nasledovat cislo metriky (euklid. - 1, kosin. - 2)." <<endl;
                exit(1);
            }
            m_metricId = atoi(argValues[i]);

  			i++;
			argCount--;
            if( argCount == 0 ) 
                   break;
		
        }

    	if( strcmp( argValues[i],"--insert") == 0 ) {

            i++;
     		--argCount;
     		cout << argValues[i] << endl;
            status = insert;
         		
         	// neni to prepinac - muze to byt jmeno souboru
         	while( ( argCount > 0 ) && ( !isSwitch(argValues[i]) )  ) {
            	// nepodarilo se mi naalokovat
            	
         		if( ( m_array = (Matrix **)realloc(m_array,sizeof(Matrix *) * (countOfFiles+1) ) ) == NULL ) {
         		
                	cout << "Chyba: Nepodarilo se alokovat pamet.";
                    exit(1);
                    
       			// podarilo se naalokovat
                } else {
        			m_array[countOfFiles] = m_pr.readPictureFromFile( argValues[i] );
                	countOfFiles++;
                	
    		    }
 	    		i++;
 	    		argCount--;
 	    		
        		if( argCount == 0 )
     				break;
            }

     		if( countOfFiles == 0 ) {
        	    cout << "Za prepinacem --insert musi byt zadany soubory." << endl;
      	    	exit(1);
     		} else {
     		   	cout << "Pocet vkladanych souboru: " << countOfFiles << endl;
     		}
     		if( argCount == 0 )
     			break;
  		}
    	if( strcmp( argValues[i],"--base") == 0 ) {
            
            substatus = base;
     		argCount--;
  			i++;
            
            b = m_bp->loadBase(argValues[i]);
            
            if( ( fileName = (char*)malloc( sizeof(char) * strlen(argValues[i] ) ) ) == NULL ) {
                cout << "Chyba: Nepodarilo se alokovat pamet pro jmeno souboru." << endl;
                exit(1);
            }
            strcpy(fileName, argValues[i]);

            if( b == NULL ) {
                status = create;
            } 

            argCount--;
            i++;

            if( argCount == 0 ) 
                break;
		}
    	if( strcmp( argValues[i],"--db") == 0 ) {
     		argCount--;
  			i++;

  			if( argCount == 0 ) {
  				cout << "Chyba: Za prepinacem --db musi nasledovat jmeno databaze." << endl;
  				exit(1);
 			}


  			if( !isSwitch(argValues[i]) ) {
                
  				dbp = new DatabasePerzistency( argValues[i] );
  				db = dbp->loadDatabase();
                if( db == NULL ) {
                    cout << "Databaze neexistuje vytvarim novou." << endl;
                    db = new Database();
                } 
  			}
     		argCount--;
  			i++;
  			if( argCount == 0 )
     			break;
		}
    	if( strcmp( argValues[i],"--create") == 0 ) {

            status = create;
    		i++;
     		--argCount;

         	// neni to prepinac - muze to byt jmeno souboru
         	while( ( argCount > 0 ) && ( !isSwitch(argValues[i]) )  ) {

            	// nepodarilo se mi naalokovat

         		if( ( m_array = (Matrix **)realloc(m_array,sizeof(Matrix *) * (countOfFiles+1) ) ) == NULL ) {
         		
                	cout << "Chyba: Nepodarilo se alokovat pamet.";
                    exit(1);
                    
       			// podarilo se naalokovat
                } else {
                
        			m_array[countOfFiles] = m_pr.readPictureFromFile( argValues[i] );
                	countOfFiles++;
                	
    		    }
 	    		i++;
 	    		argCount--;
 	    		
        		if( argCount == 0 )
     				break;
            }

     		if( countOfFiles == 0 ) {
        	    cout << "Za prepinacem --create musi byt zadany soubory." << endl;
      	    	exit(1);
     		} else {
     		   	cout << "Pocet vkladanych souboru: " << countOfFiles << endl;
     		}
     		if( argCount == 0 )
     			break;
		}
    	if( strcmp( argValues[i],"--find") == 0 ) {

    		status = find;
  			argCount--;
  			i++;
  			if( argCount == 0 ) {
  				cout << "Chyba: Za prepinacem --find musi nasledovat obrazek." << endl;
  				exit(1);
 			}
 			if( !isSwitch( argValues[i] ) ) {
      		    m = m_pr.readPictureFromFile( argValues[i] );
 			}
            argCount--;
            i++;
            if( argCount == 0 ) 
                break;

 			
		}
	}

	if( status == find ) {

   		unsigned int count = 0;

        if( b == NULL ) {
            cerr << "CHYBA: Musite zadat bazi" << endl;
            exit(1);
        }
        m = b->compareMatrixWithBase(m->getAsVector(),fileName);
        m->normalize();
        m_array = db->selectMatrix(m, m_th , count);
        
        cout << "Vyhledano " << count << " zaznamu." << endl;
   		for( unsigned int k = 0; k < count ; k++ ) {
			m_array[k]->print();
		}
	}

	if( status == insert ) {
        
        int metricId = db->getMetricId();
        if( metricId == -1 ) {
            if( m_metricId !=  -1 ) {
                if( m_metricId == 1 ) db->setMetric( em );
                else db->setMetric( cm );
            } else {
                cout << "UPOZORNENI: Neni urcena metrika. Bude pouzita standardni - euklidovska." << endl;
                db->setMetric( em );
            }
        } 

        if( b == NULL ) {
            cerr << "CHYBA: Pri vkladani do databaze musite nejprve vytvorit Basi, ktera bude ";
            cerr << "zmensovat vektor. " << endl;
            exit(1);
        }

		for( int k = 0 ; k < countOfFiles ; k++ ) {
        
            Matrix *tmp = m_array[k]->getAsVector();
            m = b->compareMatrixWithBase( tmp, fileName );
            m->normalize();
            db->addMatrix( m );
            delete tmp;
		}
        
        dbp->saveDatabase(db);

	}
	if( status == create ) {
        if( substatus == base ) {
            if( b == NULL ) {
                b = new Base();
            }
            FILE *f = NULL;
            if( ( f = fopen("vectors.r","r") ) == NULL ) {
                b->createBaseBeforeR(m_array, countOfFiles, "kov_mat.dat");
                cout << "Nyni je cas spustit R-project. Bez nej nebude program fungovat." << endl;
            // zavola se r
            } else {
                fclose(f);
            }
        } 
	}
}
/** Metoda pro urceni, zda je dany argument prepinac.
  * @version 1.2.2004
  *
  * @acces private
  * @param *value -> ukazatel na hodnotu argumentu
  * @return bool -> vraci TRUE, je-li argument prepinac, jinak vraci FALSE
 */
bool UserInterface::isSwitch(char *value) {

	bool ret = false;
	
	if( !strcmp(value, "--find"   ) ||
 		!strcmp(value, "--create" ) ||
    	!strcmp(value, "--db"     ) ||
     	!strcmp(value, "--base"   ) ||
      	!strcmp(value, "--insert" ) ||
        !strcmp(value, "--help"   ) ||
        !strcmp(value, "--th"     ) ||
        !strcmp(value, "--metric" ) ) 
        
        ret = true;
 
    return ret;
}

